#include "interval_exchange.h"

IntervalExchange::IntervalExchange(int num_intervals,
				   const FieldElement interval_widths[],
				   const int permutaion[],
				   const FieldElement holonomy[]) {
  FieldElement right_starting_points[num_intervals];
  FieldElement sum;
  for (int i=0; i<num_intervals; i++) {
    right_starting_points[i]=sum;
    //cout << "right_" << i << "= " << sum << endl;
    sum=sum+interval_widths[permutaion[i]];
  }
  
  sum=FieldElement(); //reset sum = zero
  for (int i=0; i<num_intervals; i++) {
    Interval I(sum,sum+interval_widths[i]);
    //cout << "left_" << i << "= " << sum << endl;
    TransformData td;
    td.shift=right_starting_points[permutaion[i]]-sum;
    //cout << "shift_" << i << "= " << td.shift << endl;
    
    td.holonomy=holonomy[i];
    td.color=0;
    insert(pair<Interval, TransformData>(I,td));
    sum=sum+interval_widths[i];
  }
}

// image of x under the interval exchange transformation
FieldElement IntervalExchange::operator()(const FieldElement &x) const {
  IEmap::const_iterator it=upper_bound(Interval(x,x));
  it--;
  //cout << "found :" << (*it).first[0] << endl;
  return x+(*it).second.shift;
}

// splits the interval exchange map at x so that x becomes the endpoints
// for a pair of intervals of the map
// if the split is successful return true
// if x is already an endpoint return false
bool IntervalExchange::split(const FieldElement &x) {
  IEmap::iterator it=upper_bound(Interval(x,x));
  it--;
  if ( ((*it).first[0]==x) || ((*it).first[1]==x) ) {
    return false;
  }
  // record the information from the interval we are splitting
  TransformData td=(*it).second;
  FieldElement
    start=(*it).first[0],
    end=(*it).first[1];  
  
  // remove the interval
  erase(it);
  // insert the new intervals with the same shift
  insert(pair<Interval, TransformData>(Interval(start,x),td));
  insert(pair<Interval, TransformData>(Interval(x,end),td));
  return true;
}

void IntervalExchange::iteritively_split(){

  // determine the endpoints of all the intervals.
  int num=size();
  FieldElement endpoints[num];
  int count=0;
  for (IEmap::const_iterator it=begin();	 
       it!=end();
       it++){
    endpoints[count]=(*it).first[0];
    count++;
  }
  
  for (int i=0; i<num; i++) {
    FieldElement y=operator()(endpoints[i]);
    while (split(y)) {
      y=operator()(y);
    }
  }
}
  
void IntervalExchange::color_code() {
  int color=1;
  for (IEmap::iterator it=begin();	 
       it!=end();
       it++){
    if ((*it).second.color==0) {
      IEmap::iterator j=it;
      cout << "Annulus " << color << ":" << endl;
      cout << "width= " << (*j).first.width() << endl;
      FieldElement length;
      while ((*j).second.color==0) {
	length=length+(*j).second.holonomy;
	(*j).second.color=color;
	FieldElement start=(*j).first[0]+(*j).second.shift;
	j=find(Interval(start,start));
	//cout << start << endl;
	//cout << "*j=" << (*j).first[0] << endl;
      }
      cout << "length= " << length << endl;
      color++;
    }
  }
}
  
