#ifndef INTERVAL_EXCHANGE_H
#define INTERVAL_EXCHANGE_H

#include "field.h"
#include <map>
#include <cln/real.h>
using namespace std;


class Interval {
 private:
  FieldElement x[2];
 public:
  Interval(){}
  Interval(FieldElement a, FieldElement b){
    x[0]=a;
    x[1]=b;
  }
  inline FieldElement &operator[](int i) 
    { return x[i]; }
  inline FieldElement operator[](int i) const
    { return x[i]; }
  inline FieldElement width() const 
    { return x[1]-x[0]; }
};  

// we say interval I is less than interval J
// if the absolute value of the starting point of I is less than 
// the absolute value of the starting point of J
struct IntervalComparison {
  bool operator()(const Interval &I, const Interval &J) const
  {
    using namespace cln;
    cl_R
      zI=abs(cl_N(I[0])),
      zJ=abs(cl_N(J[0]));
    return (zI < zJ);
  }
};

struct TransformData {
  FieldElement shift;
  FieldElement holonomy;
  int color;
};

typedef map<Interval, TransformData, IntervalComparison> IEmap;

class IntervalExchange : public IEmap {
 public:
  IntervalExchange(int num_intervals,
		   const FieldElement interval_widths[],
		   const int permutaion[],
		   const FieldElement holonomy[]);

  // image of x under the interval exchange transformation
  FieldElement operator()(const FieldElement &x) const;

  // splits the interval exchange map at x so that x becomes the endpoints
  // for a pair of intervals of the map
  // if the split is successful return true
  // if x is already an endpoint return false
  bool split(const FieldElement &x); 

  void iteritively_split();

  void color_code();

};

#endif
