#include "other.h"

#include <iostream>
#include <fstream>
#include <cln/real_io.h>

using namespace std;

cl_R saddleLength(int j, const IntervalExchange &ie,
		  const FieldElement &orthogonal) {
  FieldElement singularities[9];
  for (int i=0; i<9; i++) {
    singularities[i]=leftside[i].vectorProjection(orthogonal);
  }
  
  FieldElement x=rightside[j].vectorProjection(orthogonal);
  cl_R ret=length_of_level(orthogonal, x);
  bool stop=false;
  while (1) {
    for (int i=0; i<9; i++) {
      if (x==singularities[i]) 
	goto stop;
    }
    x=ie(x);   
    ret=ret+length_of_level(orthogonal, x);
  }
 stop: return ret;    
}

void printSaddleLengths(const IntervalExchange &ie,
			const FieldElement &orthogonal) {
  cl_R length_list[7];
  for (int i=1; i<8; i++) {
    length_list[i-1]=saddleLength(i,ie,orthogonal);
  }

  // now sort
  for (int i=0; i<7; i++) {
    for (int j=i+1; j<7; j++) {
      if (length_list[j]<length_list[i]) {
	cl_R temp=length_list[i];
	length_list[i]=length_list[j];
	length_list[j]=temp;
      }
    }
  }

  for (int i=1; i<7; i++) {
    length_list[i]=length_list[i]/length_list[0];
  }
  length_list[0]=length_list[0]/length_list[0];

  for (int i=0; i<7; i++) {
    cout << "Saddle Length " << i << " : " << length_list[i] << endl;
  }
}

void writePostscript(char *filename,
		     const IntervalExchange &ie,
		     const FieldElement &orthogonal) {
  ofstream file;
  file.open(filename, ios::out);
  file << "%!" << endl;
  file << 4.25*72 << " " << 5.5*72 << " translate" << endl;
  file << 200 << " " << 200 << " scale" << endl;
  file << 0.5/2.5/72 << " setlinewidth" << endl;

  for (IEmap::const_iterator it=ie.begin();
       it!=ie.end();
       it++) {
    file << "newpath" << endl;
    cl_N a=cl_N(left_endpoint(orthogonal,(*it).first[0]));
    file << (double_approx(realpart(a))) << " " 
	 << (double_approx(imagpart(a))) << " moveto" << endl;
    a=cl_N(left_endpoint(orthogonal,(*it).first[1]));
    file << (double_approx(realpart(a))) << " " 
	 << (double_approx(imagpart(a))) << " lineto" << endl;
    a=cl_N(right_endpoint(orthogonal,(*it).first[1]));
    file << (double_approx(realpart(a))) << " " 
	 << (double_approx(imagpart(a))) << " lineto" << endl;
    a=cl_N(right_endpoint(orthogonal,(*it).first[0]));
    file << (double_approx(realpart(a))) << " " 
	 << (double_approx(imagpart(a))) << " lineto" << endl;
    file << "closepath" << endl;
    file << "gsave" << endl;
    switch ((*it).second.color) {
    case 1: 
      file << ".5 .88 .28 setrgbcolor" << endl;
      break;
    case 2: 
      file << ".95 .39 .24 setrgbcolor" << endl;
      break;
    case 3: 
      file << ".31 .60 .92 setrgbcolor" << endl;
      break;
    case 4: 
      file << ".777 .14 .738 setrgbcolor" << endl;
      break;
    } 
    file << "fill" << endl;
    file << "grestore" << endl;
    file << "0 0 0 setrgbcolor" << endl;
    file << "stroke" << endl;
    
  }

  file << "showpage" << endl;
  file.close();
}
