#include "surface.h"
#include "interval_exchange.h"
#include <cln/real.h>
#include <cln/real_io.h>

// the vertices running up the left side
const FieldElement leftside[9]=
  {FieldElement(1,9)-FieldElement(1,9),
   FieldElement(1,8)-FieldElement(1,9),
   FieldElement(1,7)-FieldElement(1,9),
   FieldElement(1,7)+FieldElement(1,3)-FieldElement(1,2)-FieldElement(1,9),
   FieldElement(1,6)-FieldElement(1,9),
   FieldElement(1,6)+FieldElement(1,2)-FieldElement(1,1)-FieldElement(1,9),
   FieldElement(1,5)-FieldElement(1,9),
   FieldElement(1,4)-FieldElement(1,9),
   FieldElement(1,3)-FieldElement(1,9)};

// the vertices running up the left side
const FieldElement rightside[9]=
  {FieldElement(1,9)-FieldElement(1,9),
   FieldElement(1,10)-FieldElement(1,9),
   FieldElement(1,11)-FieldElement(1,9),
   FieldElement(1,11)+FieldElement(1,3)-FieldElement(1,4)-FieldElement(1,9),
   FieldElement(1,12)-FieldElement(1,9),
   FieldElement(1,12)+FieldElement(1,4)-FieldElement(1,5)-FieldElement(1,9),
   FieldElement(1,1)-FieldElement(1,9),
   FieldElement(1,2)-FieldElement(1,9),
   FieldElement(1,3)-FieldElement(1,9)};
   
// permuation that would take the left side to the right
const int permutation[8]={5,3,7,1,6,0,4,2};

// translation that would take an edge on the left to the glued up edge on the right
const FieldElement holonomy[8]=
  {FieldElement(1,1)-FieldElement(1,8),
   FieldElement(1,0)-FieldElement(1,7),
   FieldElement(1,2)-FieldElement(1,7),
   FieldElement(1,11)-FieldElement(1,6),
   FieldElement(1,1)-FieldElement(1,6),
   FieldElement(1,10)-FieldElement(1,5),
   FieldElement(1,0)-FieldElement(1,5),
   FieldElement(1,11)-FieldElement(1,4)};
  
FieldElement left_endpoint(const FieldElement &orthogonal,
			   const FieldElement &level){
  cl_R level_abs=abs(cl_N(level));
  int i=0;
  FieldElement lower, upper;
  cl_R sing_abs=abs(cl_N(upper=leftside[i].vectorProjection(orthogonal)));
  while ((sing_abs<=level_abs)&&(i!=8)) {
    lower=upper;
    i++;
    sing_abs=abs(cl_N(upper=leftside[i].vectorProjection(orthogonal)));  
  }
  i--;
  return 
    (level-lower)/(upper-lower)*(leftside[i+1]-leftside[i])+leftside[i];
}

FieldElement right_endpoint(const FieldElement &orthogonal,
			   const FieldElement &level){
  cl_R level_abs=abs(cl_N(level));
  int i=0;
  FieldElement lower, upper;
  cl_R sing_abs=abs(cl_N(upper=rightside[i].vectorProjection(orthogonal)));
  while ((sing_abs<=level_abs)&&(i!=8)) {
    lower=upper;
    i++;
    sing_abs=abs(cl_N(upper=rightside[i].vectorProjection(orthogonal)));  
  }
  i--;
  return 
    (level-lower)/(upper-lower)*(rightside[i+1]-rightside[i])+rightside[i];
}
